#include <tchar.h>
#include <windows.h>

#include <vector>
#include <map>

#include "support.h"
#include "zipfs/zipfs.h"
#include "wavemix/wavemix.h"
#include "zipwave.h"

#include "Scene.h"
#include "Painter.h"
#include "FontGlyph.h"

#include "Globals.h"

#include "BannerScene.h"
#include "TitleScene.h"


enum BannerGlyphs {
	BG_AKEOME = 0xE060,
	BG_YORO = 0xE061,
};

static const float AUTOPON = 3.0f;
static const float PONWAIT = 5.0f;
static const float FADEDUR = 0.8f;
static const float BLACKWAIT = 0.3f;

static BannerSceneBuilder g_bannerSceneBuilder;

BannerScene::BannerScene(int level) : Scene(level) {
	m_ticked = false;
	m_presec = 0.0f;
	m_ponsec = 0.0f;

	m_wiyo = unzLoadWave("iyo.wav");
	m_wpon = unzLoadWave("pon.wav");
}

BannerScene::~BannerScene() {
	wmUnloadWave(m_wiyo);
	wmUnloadWave(m_wpon);
}

void BannerScene::tick() {
	float sec = getsec();

	if(!m_ticked && m_ponsec == 0.0f/*m_ponsec != 0.0f can happen with very fast clicking...*/) {
		m_ticked = true;
		wmPlayWave(m_wiyo);
	}

	if(m_ponsec == 0.0f) {
		if(m_presec <= AUTOPON && AUTOPON < sec) {
			m_ponsec = sec;
			// wmKillWave(m_wiyo); // AUTOPON should be enough to play all
			wmPlayWave(m_wpon);
		}
	} else {
		if(m_ponsec + PONWAIT + FADEDUR + BLACKWAIT < sec) {
			setNextScene(new TitleScene(0));
		}
	}

	m_presec = sec;
}

void BannerScene::draw(Painter &p) {
	PenBrushRAII pbraii(p);
	float sec = getsec();
	Glyph *g;
	int fade = 255;

	if(m_ponsec != 0.0f && m_ponsec + PONWAIT < sec) {
		float f1 = std::max(std::min(1.0f - (sec - (m_ponsec + PONWAIT)) / FADEDUR, 1.0f), 0.0f);
		fade = static_cast<int>(f1 * 255);
		pbraii.on(3, RGB(fade, fade, fade), PS_SOLID, false, 0);
	}

	g = g_font.glyph(BG_AKEOME);
	g->draw(p, Point(0, 0), 1.0f, GLYPHRAND);

	if(m_ponsec != 0.0f) {
		g = g_font.glyph(BG_YORO);
		g->draw(p, Point(0, 0), 1.0f, GLYPHRAND);

		PenBrushRAII pbraii(p, 2, RGB(fade, fade, fade), PS_SOLID, false, 0);
		Point pt(70, 140);
		g_font.draw(p, pt, 0.4f, 1, L"Thanks to");       pt += Point(30, 40);
		g_font.draw(p, pt, 0.4f, 1, L"ま堂");            pt += Point(0, 25);
		g_font.draw(p, pt, 0.4f, 1, L"SoundBible.com");  pt += Point(0, 25);
		g_font.draw(p, pt, 0.4f, 1, L"SoundFX Now!");    pt += Point(0, 25);
		g_font.draw(p, pt, 0.4f, 1, L"On-Jin ～音人～"); pt += Point(0, 25);
		g_font.draw(p, pt, 0.4f, 1, L"魔王魂");          pt += Point(0, 25);
		g_font.draw(p, pt, 0.4f, 1, L"MusMus");          pt += Point(0, 25);
		g_font.draw(p, pt, 0.4f, 1, L"Senses Circuit");  pt += Point(0, 25);
		g_font.draw(p, pt, 0.4f, 1, L"WEB WAVE LIB");    pt += Point(0, 25);
		g_font.draw(p, pt, 0.4f, 1, L"くらげ工匠");      pt += Point(0, 25 + 15);
		g_font.draw(p, pt, 0.4f, 1, L"and Family");      pt += Point(0, 25);
		g_font.draw(p, pt, 0.4f, 1, L"and You!");        pt += Point(0, 25);
	}
}

void BannerScene::button(enum MouseButton btn, Point curpos, bool isdown) {
	if(!isdown) {
		return;
	}

	if(m_ponsec == 0.0f) {
		m_ponsec = getsec();
		wmKillWave(m_wiyo);
		wmPlayWave(m_wpon);
	} else {
		m_ponsec -= PONWAIT + FADEDUR + BLACKWAIT; // quick next scene
	}
}

//void BannerScene::drag(enum MouseButton btn, Point curpos) {}
//void BannerScene::hover(Point curpos) {}
