#include <tchar.h>
#include <windows.h>

#include <vector>
#include <map>

#include "support.h"
#include "zipfs/zipfs.h"
#include "wavemix/wavemix.h"
#include "zipwave.h"

#include "Scene.h"
#include "Painter.h"
#include "FontGlyph.h"

#include "Globals.h"

#include "ClassifyGameScene.h"


enum ClassifyGameGlyphs {
	CGG_TRASHBIN = 0xE070,
	CGG_BG = 0xE071,
	CGG_BIN = 0xE072,
	CGG_CAN,
	CGG_DUCK,
};

static const Point g_binpoints[3] = {
	Point( 30, 280),
	Point(235, 280),
	Point(440, 280),
};

static const int g_bintypes[3] = {
	CGG_DUCK,
	CGG_BIN,
	CGG_CAN,
};

static const float OBJAY = 0.5f;

static ClassifyGameSceneBuilder g_classifyGameSceneBuilder;

ClassifyGameScene::ClassifyGameScene(int level) : Scene(level) {
	m_maxobj = std::min(level, 3);
	m_leftobjs = m_maxobj;
	m_grabbingidx = -1;

	Point pt(0, 0);
	for(int i = 0; i < m_maxobj; i++) {
		m_objtype[i] = CGG_BIN + randp(3);
		m_objfalling[i] = false;
		m_objlanded[i] = false;

		Size bb = g_font.glyph(m_objtype[i])->bounding();
		pt += bb.onlycx().asPoint();
		if(1 <= i) {
			pt += Point(40, 0);
		}
	}
	pt = Point(320, 20) - (pt / 2);
	for(int i = 0; i < m_maxobj; i++) {
		m_objpos[i] = pt;
		Size bb = g_font.glyph(m_objtype[i])->bounding();
		pt += bb.onlycx().asPoint() + Point(40, 0);
	}

	m_wpick = unzLoadWave("ta_ta_syupon01.wav");
	m_wfall = unzLoadWave("fall02.wav");
	m_wland = unzLoadWave("HitB@11.wav");
	m_wgood = unzLoadWave("se_maoudamashii_chime14.wav");
	m_wbad = unzLoadWave("ta_ge_quiz_no01.wav");
	m_wclear = unzLoadWave("muci_fan_08.wav");
}

ClassifyGameScene::~ClassifyGameScene() {
	wmUnloadWave(m_wpick);
	wmUnloadWave(m_wfall);
	wmUnloadWave(m_wland);
	wmUnloadWave(m_wgood);
	wmUnloadWave(m_wbad);
	wmUnloadWave(m_wclear);
}

void ClassifyGameScene::tick() {
	Glyph *g;

	Size gbinbb = g_font.glyph(CGG_TRASHBIN)->bounding();
	for(int i = 0; i < m_maxobj; i++) {
		if(m_objfalling[i]) {
			m_objpos[i] += Point(m_objv[i].first, m_objv[i].second);
			m_objv[i].second += OBJAY;

			g = g_font.glyph(m_objtype[i]);
			for(int j = 0; j < 3; j++) {
				Size gbb = g->bounding();
				int boundline = g_binpoints[j].y() + gbinbb.cy() - gbb.cy();
				if(g_binpoints[j].rect(gbinbb - gbb.onlycx() / 2).has(m_objpos[i])
				   && boundline <= m_objpos[i].y()) {
					m_objpos[i].sety(boundline);
					m_objfalling[i] = false;
					m_objlanded[i] = true;
					wmPlayWave(m_wland);
					m_leftobjs--;
					if(result() == SR_PLAYING) {
						if(g_bintypes[j] == m_objtype[i]) {
							if(0 < m_leftobjs) {
								// more objects to go...
								wmPlayWave(m_wgood);
							} else {
								// complete
								wmPlayWave(m_wclear);
								setResult(SR_SUCCESS);
							}
						} else {
							wmPlayWave(m_wbad);
							setResult(SR_FAILED);
						}
					}
				}
			}
		}
	}
}

void ClassifyGameScene::draw(Painter &p) {
	Glyph *g;

	g = g_font.glyph(CGG_BG);
	g->draw(p, Point(0, 0), 1.0f, GLYPHRAND);

	g = g_font.glyph(CGG_TRASHBIN);
	for(int i = 0; i < 3; i++) {
		g->draw(p, g_binpoints[i], 1.0f, GLYPHRAND);
	}

	for(int i = 0; i < m_maxobj; i++) {
		g = g_font.glyph(m_objtype[i]);
		g->draw(p, m_objpos[i], 1.0f, GLYPHRAND);
	}
}

void ClassifyGameScene::button(enum MouseButton btn, Point curpos, bool isdown) {
	if(isdown) {
		if(result() == SR_PLAYING) {
			for(int i = 0; i < m_maxobj; i++) {
				Glyph *g = g_font.glyph(m_objtype[i]);
				if(m_objlanded[i] == false && m_objpos[i].rect(g->bounding()).has(curpos)) {
					m_objfalling[i] = false;
					m_grabbingidx = i;
					m_objgraboff = m_objpos[i] - curpos;
					wmPlayWave(m_wpick);
					break;
				}
			}
		}
	} else {
		if(m_grabbingidx != -1) {
			// release
			Size obb = g_font.glyph(m_objtype[m_grabbingidx])->bounding();
			Size gbinbb = g_font.glyph(CGG_TRASHBIN)->bounding();
			for(int i = 0; i < 3; i++) {
				// TODO: consider x-axis pos/vec... 
				if(g_binpoints[i].rect(gbinbb - obb.onlycx() / 2).yaxis(0, 480).has(m_objpos[m_grabbingidx])) {
					m_objfalling[m_grabbingidx] = true;
					m_objv[m_grabbingidx].first = 0;
					m_objv[m_grabbingidx].second = (curpos - m_precurpos).y();
					wmPlayWave(m_wfall);
					break;
				}
			}
			m_grabbingidx = -1;
		}
	}

	m_precurpos = curpos;
}

void ClassifyGameScene::drag(enum MouseButton btn, Point curpos) {
	if(m_grabbingidx != -1) {
		m_objpos[m_grabbingidx] = curpos + m_objgraboff;
	}

	m_precurpos = curpos;
}
//void ClassifyGameScene::hover(Point curpos) {}
