#include <tchar.h>
#include <windows.h>

#include <vector>
#include <map>

#include "support.h"
#include "zipfs/zipfs.h"
#include "wavemix/wavemix.h"
#include "zipwave.h"

#include "Scene.h"
#include "Painter.h"
#include "FontGlyph.h"

#include "Globals.h"

#include "CrocoGameScene.h"


enum CrocoGlyphs {
	CG_MOUTHOPEN = 0xE020,
	CG_TEETH1 = 0xE021,
	CG_TEETH2,
	CG_TEETH3,
	CG_TEETH4,
	CG_PRESSED1 = 0xE025,
	CG_PRESSED2,
	CG_PRESSED3,
	CG_PRESSED4,
	CG_MOUTHCLOSE = 0xE029,
	CG_MOUTHCLOSEWINK = 0xE02A,
};

static const Rect g_teeths[] = {
	Rect(178, 305, 242, 382),
	Rect(260, 329, 325, 413),
	Rect(352, 332, 420, 405),
	Rect(433, 326, 485, 381),
};

static CrocoGameSceneBuilder g_crocoSceneBuilder;

CrocoGameScene::CrocoGameScene(int level) : Scene(level) {
	for(unsigned int i = 0; i < _countof(m_teethstate); i++) {
		m_teethstate[i] = true;
	}
	switch(level) {
	case 1:
		// 1 tooth
		m_teethstate[randp(_countof(m_teethstate))] = false;
		break;
	case 2:
		// 2 teeth
		m_teethstate[randp(_countof(m_teethstate))] = false;
		for(;;) {
			int i = randp(_countof(m_teethstate));
			if(m_teethstate[i] == true) {
				m_teethstate[i] = false;
				break;
			}
		}
		break;
	default: // ...?
		// FALLTHROUGH
	case 3:
		// all teeth
		for(unsigned int i = 0; i < _countof(m_teethstate); i++) {
			m_teethstate[i] = false;
		}
		break;
	}

	m_complete = false;
	m_ticked = false;

	m_bite = unzLoadWave("Cartoon-TeethChomp.wav");
	m_bgm = unzLoadWave("muci_keikai_06.wav");
	m_switch = unzLoadWave("sei_ge_sui01.wav");
}

CrocoGameScene::~CrocoGameScene() {
	wmUnloadWave(m_bite);
	wmUnloadWave(m_bgm);
}

void CrocoGameScene::tick() {
	if(!m_ticked) {
		m_ticked = true;
		wmPlayWave(m_bgm);
	}

	if(2.5f < getsec() && m_complete) {
		setResult(SR_SUCCESS);
	}
}

void CrocoGameScene::draw(Painter &p) {
	Glyph *g;

	if(!m_complete) {
		g = g_font.glyph(CG_MOUTHOPEN);
		g->draw(p, Point(0, 0), 1.0f, GLYPHRAND);

		for(unsigned int i = 0; i < _countof(m_teethstate); i++) {
			g = g_font.glyph(m_teethstate[i] ? CG_PRESSED1 + i : CG_TEETH1 + i);
			g->draw(p, Point(0, 0), 1.0f, GLYPHRAND);
		}
	} else {
		g = g_font.glyph(m_wink ? CG_MOUTHCLOSEWINK : CG_MOUTHCLOSE);
		g->draw(p, Point(0, 0), 1.0f, GLYPHRAND);
	}
}

void CrocoGameScene::button(enum MouseButton btn, Point curpos, bool isdown) {
	if(!isdown) {
		return;
	}

	if(m_complete) {
		return;
	}

	for(unsigned int i = 0; i < _countof(m_teethstate); i++) {
		if(g_teeths[i].has(curpos)) {
			m_teethstate[i] = true;

			wmPlayWave(m_switch);

			m_complete = true;
			for(unsigned int i = 0; i < _countof(m_teethstate); i++) {
				m_complete &= m_teethstate[i];
			}

			if(m_complete) {
				wmPlayWave(m_bite);
				m_wink = (randp(10) == 0); // probability=1/10
			}

			break;
		}
	}
}

//void TemplateScene::drag(enum MouseButton btn, Point curpos) {}
//void TemplateScene::hover(Point curpos) {}
