#include <tchar.h>
#include <windows.h>

#include <vector>
#include <map>

#include "support.h"
#include "zipfs/zipfs.h"
#include "wavemix/wavemix.h"
#include "zipwave.h"

#include "Scene.h"
#include "Painter.h"
#include "FontGlyph.h"

#include "Globals.h"

#include "EndrollScene.h"
#include "TitleScene.h"


enum EndrollGlyphs {
};

static const float SCROLLSEC = 9.0f;

static EndrollSceneBuilder g_endrollSceneBuilder;

EndrollScene::EndrollScene(int level) : Scene(level) {
	m_ticked = false;

	m_wbgm = unzLoadWave("st018.wav");
}

EndrollScene::~EndrollScene() {
	wmUnloadWave(m_wbgm);
}

void EndrollScene::tick() {
	if(!m_ticked) {
		m_ticked = true;
		wmPlayWave(m_wbgm);
	}

	if(SCROLLSEC < getsec()) {
		setNextScene(new TitleScene(0));
	}
}

static void centerText(Painter &p, Point &pos, float mag, int rand, LPCWSTR str) {
	Size bb = g_font.bounding(mag, str);
	g_font.draw(p, pos - bb.asPoint() / 2, mag, rand, str);
	pos += bb.onlycy().asPoint() + Point(0, 10);
}

void EndrollScene::draw(Painter &p) {
	Point pos(320, 480 + 200 - static_cast<int>(getsec() * 450.0f));

	static const float mag_h1 = 1.0f;
	static const float mag_h2 = 0.7f;
	static const float mag_h3 = 0.5f;
	static const float mag_h4 = 0.3f;

	centerText(p, pos, mag_h1, 2, L"Staff");
	pos += Point(0, 80);

	centerText(p, pos, mag_h2, 2, L"プロデュ～サ～");
	pos += Point(0, 30);
	centerText(p, pos, mag_h3, 2, L"むらちゅえ");
	pos += Point(0, 50);

	centerText(p, pos, mag_h2, 2, L"ディレクタ～");
	pos += Point(0, 30);
	centerText(p, pos, mag_h3, 2, L"むらちゅえ");
	pos += Point(0, 50);

	centerText(p, pos, mag_h2, 2, L"プログラマ～");
	pos += Point(0, 30);
	centerText(p, pos, mag_h3, 2, L"むらちゅえ");
	pos += Point(0, 50);

	centerText(p, pos, mag_h2, 2, L"絵");
	pos += Point(0, 30);
	centerText(p, pos, mag_h3, 2, L"むらちゅえ");
	pos += Point(0, 50);

	centerText(p, pos, mag_h2, 2, L"アイデア");
	pos += Point(0, 30);
	centerText(p, pos, mag_h3, 2, L"むらちゅえ");
	centerText(p, pos, mag_h3, 2, L"あ～ちゃん");
	pos += Point(0, 50);

	centerText(p, pos, mag_h2, 2, L"テストプレイ");
	pos += Point(0, 30);
	centerText(p, pos, mag_h3, 2, L"むらちゅえ");
	centerText(p, pos, mag_h3, 2, L"あ～ちゃん");
	pos += Point(0, 50);

	centerText(p, pos, mag_h2, 2, L"QA");
	pos += Point(0, 30);
	centerText(p, pos, mag_h3, 2, L"むらちゅえ");
	pos += Point(0, 50);

	centerText(p, pos, mag_h2, 2, L"効果音・BGM");
	pos += Point(0, 30);
	centerText(p, pos, mag_h3, 2, L"ま堂");
	centerText(p, pos, mag_h4, 2, L"http://kizoku.tyabo.com/mus/");
	pos += Point(0, 20);
	centerText(p, pos, mag_h3, 2, L"SoundBible.com");
	centerText(p, pos, mag_h4, 2, L"http://soundbible.com/");
	pos += Point(0, 20);
	centerText(p, pos, mag_h3, 2, L"SoundFX Now!");
	centerText(p, pos, mag_h4, 2, L"http://soundfxnow.com/");
	pos += Point(0, 20);
	centerText(p, pos, mag_h3, 2, L"On-Jin ～音人～");
	centerText(p, pos, mag_h4, 2, L"http://on-jin.com/");
	pos += Point(0, 20);
	centerText(p, pos, mag_h3, 2, L"魔王魂");
	centerText(p, pos, mag_h4, 2, L"http://maoudamashii.jokersounds.com/");
	pos += Point(0, 20);
	centerText(p, pos, mag_h3, 2, L"フリーBGM・音楽素材 MusMus");
	centerText(p, pos, mag_h4, 2, L"http://musmus.main.jp/");
	pos += Point(0, 20);
	centerText(p, pos, mag_h3, 2, L"フリー音楽素材 Senses Circuit");
	centerText(p, pos, mag_h4, 2, L"http://www.senses-circuit.com/");
	pos += Point(0, 20);
	centerText(p, pos, mag_h3, 2, L"WEB WAVE LIB 音と声のonline素材集");
	centerText(p, pos, mag_h4, 2, L"http://wwl.s-t-t.com/");
	pos += Point(0, 20);
	centerText(p, pos, mag_h3, 2, L"フリー効果音素材 くらげ工匠");
	centerText(p, pos, mag_h4, 2, L"http://www.kurage-kosho.info/guide.html");
	pos += Point(0, 20);
	centerText(p, pos, mag_h3, 2, L"むらちゅえ");
	centerText(p, pos, mag_h4, 2, L"http://murachue.ddo.jp/");
	pos += Point(0, 50);

	pos += Point(0, 30);
	centerText(p, pos, mag_h4, 2, L"© 2013 Murachue.");
	centerText(p, pos, mag_h4, 2, L"All rights reserved.");
	pos += Point(0, 50);

	pos += Point(0, 200);
	{
		PenBrushRAII pbraii(p, 2, RGB(255, 255, 255), PS_SOLID, false, 0);
		centerText(p, pos, mag_h3, 2, L"2014年もよろしくおねがいいたします～");
	}

	{
		float f1 = std::min(std::max(((SCROLLSEC - getsec()) - 1.0f) / 0.7f, 0.0f), 1.0f);
		int fade = f1 * 255;
		PenBrushRAII pbraii(p, 2, RGB(fade, fade, fade), PS_SOLID, false, 0);
		LPCWSTR str = L"Ctrlおしつづけながらクリックですた～とすると…?";
		float mag = 0.3f;
		Size sz = g_font.bounding(mag, str);
		g_font.draw(p, Point(640 - 10, 480 - 10) - sz.asPoint(), 0.3f, 1, str);
	}
}

//void EndrollScene::button(enum MouseButton btn, Point curpos, bool isdown) {}

//void EndrollScene::drag(enum MouseButton btn, Point curpos) {}
//void EndrollScene::hover(Point curpos) {}
