#include <tchar.h>
#include <windows.h>

#include <stdio.h> // TODO: use istream/ostream?

#include <vector>
#include <map>
//#include <algorithm> // for max

#include "support.h"
#include "zipfs/zipfs.h"

#include "Painter.h"
#include "FontGlyph.h"

#include "Globals.h"

const int GLYPHRAND = 3;

const void *Glyph::load(const void *data) {
	// TODO: reduce this too-duplicated codes...
	const short width = *reinterpret_cast<const short*>(data);
	data = reinterpret_cast<const short*>(data) + 1;
	const short height = *reinterpret_cast<const short*>(data);
	data = reinterpret_cast<const short*>(data) + 1;

	m_bounding = Size(width, height);

	const unsigned short count = *reinterpret_cast<const short*>(data);
	data = reinterpret_cast<const short*>(data) + 1;

	m_pts.clear();

	for(unsigned int i = 0; i < count; i++) {
		const short x = *reinterpret_cast<const short*>(data);
		data = reinterpret_cast<const short*>(data) + 1;
		const short y = *reinterpret_cast<const short*>(data);
		data = reinterpret_cast<const short*>(data) + 1;
		const bool np = *reinterpret_cast<const char*>(data) != 0 ? true : false;
		data = reinterpret_cast<const char*>(data) + 1;

		m_pts.push_back(GlyphPoint(Point(x, y), np));
	}

	return data;
}

const void *Glyph::loadText(const void *data) {
	// TODO: reduce this too-duplicated codes...
	int width;
	int height;

	sscanf(reinterpret_cast<const char*>(data), "%d,%d", &width, &height);
	data = strchr(reinterpret_cast<const char*>(data), '\n') + 1;

	m_bounding = Size(width, height);

	unsigned int count;
	sscanf(reinterpret_cast<const char*>(data), "%u", &count);
	data = strchr(reinterpret_cast<const char*>(data), '\n') + 1;

	m_pts.clear();

	for(unsigned int i = 0; i < count; i++) {
		int x;
		int y;
		int np;
		sscanf(reinterpret_cast<const char*>(data), "%d,%d,%d", &x, &y, &np);
		data = strchr(reinterpret_cast<const char*>(data), '\n') + 1;

		m_pts.push_back(GlyphPoint(Point(x, y), np ? true : false));
	}

	return data;
}

void Glyph::draw(Painter &p, Point pt, float mag, int randparam, bool bbox, bool isEdit, int selidx, Point *cursor) {
	// first point must be newPoint...
	std::vector<GlyphPoint>::iterator it = m_pts.begin();
	Point prept;

	if(cursor != NULL) {
		Point gcur = ((*cursor - pt) / mag * mag) + pt;
		p.frameRect(gcur.box(3));
	}

	for(int i = 0; it != m_pts.end(); it++, i++) {
		Point curpt;

		if(i == selidx && cursor != NULL) {
			Point gcur = (*cursor - pt) / mag;
			curpt = gcur * mag + pt;
		} else {
			curpt = it->first * mag + pt + Point(randnp(randparam), randnp(randparam));
		}

		if(0 < i) {
			if(it->second) {
				// newPoint
				// do nothing.
			} else {
				// not newPoint
				p.line(prept, curpt);
			}
		}
		if(isEdit) {
			// note: do not add randparam nor *cursor for Point!
			Rect box = (it->first * mag + pt).box(3);
			PenBrushRAII pbraii(p);
			if(cursor != NULL && box.has(*cursor)) {
				pbraii.on(1, RGB(255, 0, 0), PS_SOLID, false, 0);
			}
			if(i == selidx) {
				p.fillRect(box);
			} else {
				p.frameRect(box);
			}
		}
		prept = curpt;
	}

	if(selidx == static_cast<int>(m_pts.size())) {
		// will not new-point but last
		GlyphPoint &last = m_pts[m_pts.size() - 1];
		Point gcur = (*cursor - pt) / mag;
		p.line(last.first * mag + pt, gcur * mag + pt);
	}
	if(bbox) {
		// bounding box
		p.frameRect(pt.rect(m_bounding * mag));
	}
}

unsigned int Glyph::pointlen() {
	return m_pts.size();
}

const GlyphPoint *Glyph::point(unsigned int idx) {
	if(m_pts.size() <= idx) {
		debug(_T("point?? size=%u <= idx=%u"), m_pts.size(), idx);
		return NULL;
	}

	return &m_pts[idx];
}

void Glyph::insPoint(unsigned int idx, Point p, bool newPoint) {
	if(m_pts.size() < idx) {
		debug(_T("inspoint?? size=%u < idx=%u"), m_pts.size(), idx);
		return;
	}

	if(m_pts.size() == idx) {
		// append
		m_pts.push_back(GlyphPoint(p, newPoint));
	} else {
		// insert
		std::vector<GlyphPoint>::iterator it = m_pts.begin();
		std::advance(it, idx);
		m_pts.insert(it, GlyphPoint(p, newPoint));
	}

	updateWH();
}

void Glyph::setPoint(unsigned int idx, Point p, bool newPoint) {
	if(m_pts.size() <= idx) {
		debug(_T("setpoint?? size=%u <= idx=%u"), m_pts.size(), idx);
		return;
	}

	m_pts[idx] = GlyphPoint(p, newPoint);

	updateWH();
}

void Glyph::delPoint(unsigned int idx) {
	if(m_pts.size() < idx) {
		debug(_T("delpoint?? size=%u <= idx=%u"), m_pts.size(), idx);
		return;
	}

	std::vector<GlyphPoint>::iterator it = m_pts.begin();
	std::advance(it, idx);
	m_pts.erase(it);

	updateWH();
}

void Glyph::clearPoints() {
	m_pts.clear();
	updateWH();
}

void Glyph::updateWH() {
	std::vector<GlyphPoint>::iterator it = m_pts.begin();
	Rect rc(0, 0, 0, 0);
	for(; it != m_pts.end(); it++) {
		if(it->first.x() < rc.left()) { rc.setLeft(it->first.x()); }
		if(it->first.y() < rc.top()) { rc.setTop(it->first.y()); }
		if(rc.right() < it->first.x()) { rc.setRight(it->first.x()); }
		if(rc.bottom() < it->first.y()) { rc.setBottom(it->first.y()); }
	}
	m_bounding = rc.size();
}

void Glyph::dump(FILE *fp) {
	short width = m_bounding.cx();
	short height = m_bounding.cy();
	fwrite(&width, sizeof(width), 1, fp);
	fwrite(&height, sizeof(height), 1, fp);
	unsigned short size = m_pts.size();
	fwrite(&size, sizeof(size), 1, fp);
	for(unsigned int i = 0; i < size; i++) {
		{
			short b;
			b = m_pts[i].first.x(); fwrite(&b, sizeof(b), 1, fp);
			b = m_pts[i].first.y(); fwrite(&b, sizeof(b), 1, fp);
		}
		{
			char b;
			b = m_pts[i].second; fwrite(&b, sizeof(b), 1, fp);
		}
	}
}

void Glyph::dumpText(FILE *fp) {
	fprintf(fp, "%ld,%ld\n", m_bounding.cx(), m_bounding.cy());
	fprintf(fp, "%Iu\n", m_pts.size());
	for(unsigned int i = 0; i < m_pts.size(); i++) {
		fprintf(fp, "%ld,%ld,%d\n", m_pts[i].first.x(), m_pts[i].first.y(), m_pts[i].second ? 1 : 0);
	}
}

Point Glyph::findRealTopLeft() {
	Point tl = m_bounding.asPoint();
	for(std::vector<GlyphPoint>::iterator it = m_pts.begin(); it != m_pts.end(); it++) {
		if(it->first.x() < tl.x()) { tl.setx(it->first.x()); }
		if(it->first.y() < tl.y()) { tl.sety(it->first.y()); }
	}
	return tl;
}

void Glyph::justify(Point pt) {
	Point tl = findRealTopLeft();
	tl = tl - pt; // offset
	if(tl != Point(0, 0)) {
		for(std::vector<GlyphPoint>::iterator it = m_pts.begin(); it != m_pts.end(); it++) {
			it->first -= tl;
		}
		updateWH();
	}
}

void Glyph::setGlyph(Glyph &g) {
	m_pts = g.m_pts;
	m_bounding = g.m_bounding;
}

void Glyph::scale(float mag) {
	for(std::vector<GlyphPoint>::iterator it = m_pts.begin(); it != m_pts.end(); it++) {
		it->first = it->first * mag;
	}
	updateWH();
}

void Font::load(const void *data) {
	m_glyphs.clear();

	for(;;) {
		const wchar_t ch = *reinterpret_cast<const wchar_t *>(data);
		data = reinterpret_cast<const wchar_t *>(data) + 1;
		if(ch == 0) {
			break;
		}
		Glyph *g = newGlyph(ch);
		data = g->load(data);
	}
}

void Font::loadText(const void *data) {
	m_glyphs.clear();

	for(;;) {
		unsigned short us_ch;
		sscanf(reinterpret_cast<const char*>(data), "%04hX", &us_ch);
		const wchar_t ch = us_ch;
		data = strchr(reinterpret_cast<const char*>(data), '\n') + 1;
		if(ch == 0) {
			break;
		}
		Glyph *g = newGlyph(ch);
		data = g->loadText(data);
	}
}

void Font::draw(Painter &p, Point pt, float mag, int randparam, LPCWSTR text, bool bbox) {
	Point orgp = pt;
	LONG ymax = 0;
	for(; *text; text++) {
		Glyph *g = glyph(*text);
		if(!g) {
			g = glyph(0xFFFD);
		}
		if(g) {
			g->draw(p, pt, mag, randparam);
			Size gsz = g->bounding(mag);
			pt.setx(pt.x() + gsz.cx());
			if(ymax < gsz.cy()) {
				ymax = gsz.cy();
			}
		}
	}
	if(bbox) {
		p.frameRect(orgp.bottomright(Point(pt.x(), orgp.y() + ymax)));
	}
}

Size Font::bounding(float mag, LPCWSTR str, int len) {
	if(len == -1) {
		len = wcslen(str);
	}

	LONG x = 0;
	LONG y = 0;
	for(int i = 0; i < len; i++) {
		Glyph *g = g_font.glyph(str[i]);
		if(!g) {
			g = g_font.glyph(0xFFFD);
		}
		x += g->bounding().cx() * mag;
		y = std::max(y, static_cast<LONG>(g->bounding().cy() * mag));
	}

	return Size(x, y);
}

Glyph *Font::newGlyph(wchar_t ch) {
	if(m_glyphs.count(ch) > 0) {
		// TODO: or return exists entry?
		return NULL;
	}
	return &m_glyphs[ch]; // std::map#[] creates entry if not found.
}

Glyph *Font::glyph(wchar_t ch) {
	if(m_glyphs.count(ch) == 0) {
		return NULL;
	}
	return &m_glyphs[ch];
}

void Font::dump() {
	FILE *fp;

	if((fp = fopen("data\\font.bin", "wb")) == NULL) {
		debug(_T("could not open data\\font.bin!"));
		return;
	}

	for(std::map<wchar_t, Glyph>::iterator it = m_glyphs.begin(); it != m_glyphs.end(); it++) {
		if(it->second.pointlen() == 0) {
			// garbage collect now!
			continue;
		}
		wchar_t ch = it->first;
		fwrite(&ch, sizeof(ch), 1, fp);
		it->second.dump(fp);
	}

	wchar_t ch = 0;
	fwrite(&ch, sizeof(ch), 1, fp);

	fclose(fp);
}

void Font::dumpText() {
	FILE *fp;

	if((fp = fopen("data\\font.txt", "w")) == NULL) {
		debug(_T("could not open data\\font.txt!"));
		return;
	}

	for(std::map<wchar_t, Glyph>::iterator it = m_glyphs.begin(); it != m_glyphs.end(); it++) {
		if(it->second.pointlen() == 0) {
			// garbage collect now!
			continue;
		}
		fprintf(fp, "%04x\n", it->first);
		it->second.dumpText(fp);
	}

	fprintf(fp, "%04x\n", 0);

	fclose(fp);
}

wchar_t Font::nextch(wchar_t ch) {
	std::map<wchar_t, Glyph>::iterator it;

	for(it = m_glyphs.begin(); it != m_glyphs.end(); it++) {
		if(it->first == ch) {
			it++;
			if(it != m_glyphs.end()) {
				return it->first;
			}
			// this is last one...
			it = m_glyphs.begin();
			if(it != m_glyphs.end()) {
				return it->first;
			}
			break;
		}
	}

	// "ch" not found... return first one if can.
	it = m_glyphs.begin();
	if(it != m_glyphs.end()) {
		return it->first;
	}

	return L'\0';
}

wchar_t Font::prevch(wchar_t ch) {
	wchar_t prev = L'\0';

	for(std::map<wchar_t, Glyph>::iterator it = m_glyphs.begin(); it != m_glyphs.end(); it++) {
		if(it->first == ch) {
			if(it == m_glyphs.begin()) {
				continue; // till end.
			}
			return prev;
		}
		prev = it->first;
	}
	return prev; // m_glyphs.end()-1 or L'\0'(prev's initial value).
}
