// note: Alphanum Glyph should be sized (40,60).
typedef std::pair<Point, bool> GlyphPoint;
class Glyph {
public:
// ctors
	Glyph() : m_bounding(0, 0) {}
// methods
	const void *load(const void *data);
	const void *loadText(const void *data);
	void draw(Painter &p, Point pt, float mag, int randparam, bool bbox=false, bool isEdit=false, int selidx=-1, Point *cursor=NULL);
	Size bounding(float mag=1.0f) { return Size(m_bounding.cx() * mag, m_bounding.cy() * mag); }

	unsigned int pointlen();
	const GlyphPoint *point(unsigned int idx);
	void insPoint(unsigned int idx, Point p, bool newPoint);
	void setPoint(unsigned int idx, Point p, bool newPoint);
	void delPoint(unsigned int idx);
	void clearPoints();
	void dump(FILE *fp);
	void dumpText(FILE *fp);
	Point findRealTopLeft();
	void justify(Point pt);
	void setGlyph(Glyph &g);
	void scale(float mag);
private:
// variables
	std::vector<GlyphPoint> m_pts;
	Size m_bounding;
// internal functions
	void updateWH();
};

// Unicode (UCS-2/BMP) only!
class Font {
public:
// ctors
	Font() {}
// methods
	void load(const void *data);
	void loadText(const void *data);
	void draw(Painter &p, Point pt, float mag, int randparam, LPCWSTR text, bool bbox=false);
	Size bounding(float mag, LPCWSTR str, int len=-1);

	Glyph *newGlyph(wchar_t ch);
	Glyph *glyph(wchar_t ch);
	void dump();
	void dumpText();
	wchar_t nextch(wchar_t ch);
	wchar_t prevch(wchar_t ch);
private:
// variables
	std::map<wchar_t, Glyph> m_glyphs;
};

extern const int GLYPHRAND;
