#include <tchar.h>
#include <windows.h>
#include <windowsx.h> // for Message Cracker

#include <stdio.h>

#include <vector>
#include <map>

#include "support.h"
#include "zipfs/zipfs.h"
#include "wavemix/wavemix.h"
#include "resource.h"

#include "Painter.h"
#include "FontGlyph.h"

#include "Globals.h"

#include "Scene.h"
#include "MainWindow.h"

#ifdef _DEBUG
#include "SelectorScene.h"
#else
#include "BannerScene.h"
#endif

#define DOUBLEBUFFERING

// HANDLE_WM_MOUSEWHEEL is not presented by mingw32-w32api-dev 4.0.3-1 windowsx.h.
// http://read.pudn.com/downloads161/sourcecode/windows/727405/scroller.c__.htm
// pudn... china power...
#ifndef HANDLE_WM_MOUSEWHEEL
#define HANDLE_WM_MOUSEWHEEL( hwnd, wParam, lParam, fn )       \
	((fn)((hwnd), LOWORD(wParam), (int)(short) HIWORD(wParam), \
	GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) ), 0L)
#endif

void MainWindow::create(int nShowCmd) {
	WNDCLASSEX wcex = {0};

	wcex.cbSize = sizeof(wcex);
	wcex.lpfnWndProc = reinterpret_cast<WNDPROC>(_windowProc);
	wcex.cbWndExtra = sizeof(MainWindow*);
	wcex.hInstance = g_hInst;
	wcex.hIcon = reinterpret_cast<HICON>(LoadImage(g_hInst, MAKEINTRESOURCE(IDI_APPICON), IMAGE_ICON, 32, 32, LR_DEFAULTCOLOR));
	wcex.hCursor = LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground = CreateSolidBrush(RGB(0, 0, 0));
	wcex.lpszClassName = _T("AKE2014_MainWindow");
	wcex.hIconSm = reinterpret_cast<HICON>(LoadImage(g_hInst, MAKEINTRESOURCE(IDI_APPICON), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR));
	ATOM wcatom = RegisterClassEx(&wcex);
	if(wcatom == 0) die(_T("RegisterClassEx無理でした"));

	// XXX: effective on only primary display.
	//      should be "cursor-placed display"?
	Rect rc;
	GetWindowRect(GetDesktopWindow(), rc.prect());
	//debug(_T("%d,%d-%d,%d"), rc.left(), rc.top(), rc.right(), rc.bottom());
	SystemParametersInfo(SPI_GETWORKAREA, 0, rc.prect(), 0);
	//debug(_T("%d,%d-%d,%d"), rc.left(), rc.top(), rc.right(), rc.bottom());

	DWORD style = WS_OVERLAPPEDWINDOW;

	Rect wrc(0, 0, 640, 480);
	AdjustWindowRect(wrc.prect(), style, false);

	int x, y;
	x = (rc.width() - wrc.width()) / 2;
	y = (rc.height() - wrc.height()) / 2;

	m_hWnd = CreateWindowEx(0, MAKEINTATOM(wcatom), _T("Ake2014"), style, x, y, wrc.width(), wrc.height(), NULL, NULL, g_hInst, NULL);
	if(m_hWnd == 0) die(_T("CreateWindowEx無理でした"));
	SetWindowLongPtr(m_hWnd, 0, reinterpret_cast<LONG_PTR>(this));

	ShowWindow(m_hWnd, nShowCmd);

	SetTimer(m_hWnd, 1, 1000/60, NULL);

	g_unz = openres();

	long fontsize;
	BYTE *fontdata = unzReadWhole(g_unz, "font.bin", &fontsize);
	g_font.load(fontdata);
	free(fontdata);

	wmInitSystem();

	// XXX: should handled in Scene::Scene() or Scene::addBuilder().
	std::vector<SceneBuilder*> builders = g_sceneManager.allbuilders();
	for(std::vector<SceneBuilder*>::iterator it = builders.begin(); it != builders.end(); it++) {
		if(!(*it)->isDevScene()) {
			g_sceneManager.addBuilder(*it);
		}
	}

#ifdef _DEBUG
	m_scene = new SelectorScene(0);
#else
	m_scene = new BannerScene(0);
#endif
}

LRESULT CALLBACK MainWindow::windowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	switch(uMsg) {
	HANDLE_MSG(hWnd, WM_CLOSE, onClose);
	HANDLE_MSG(hWnd, WM_SIZE, onSize);
	HANDLE_MSG(hWnd, WM_PAINT, onPaint);
	HANDLE_MSG(hWnd, WM_MOUSEMOVE, onMousemove);
	HANDLE_MSG(hWnd, WM_LBUTTONDOWN, onLbuttondown);
	HANDLE_MSG(hWnd, WM_LBUTTONUP, onLbuttonup);
	HANDLE_MSG(hWnd, WM_TIMER, onTimer);
	HANDLE_MSG(hWnd, WM_RBUTTONDOWN, onRbuttondown);
	HANDLE_MSG(hWnd, WM_RBUTTONUP, onRbuttonup);
	HANDLE_MSG(hWnd, WM_KEYDOWN, onKey);
	HANDLE_MSG(hWnd, WM_KEYUP, onKey);
	HANDLE_MSG(hWnd, WM_CHAR, onChar);
	HANDLE_MSG(hWnd, WM_MBUTTONDOWN, onMbuttondown);
	HANDLE_MSG(hWnd, WM_MBUTTONUP, onMbuttonup);
	HANDLE_MSG(hWnd, WM_MOUSEWHEEL, onMousewheel);
	default:
		return DefWindowProc(hWnd, uMsg, wParam, lParam);
	}
}

LRESULT CALLBACK MainWindow::_windowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	MainWindow *mw = reinterpret_cast<MainWindow*>(GetWindowLongPtr(hWnd, 0));
	return mw->windowProc(hWnd, uMsg, wParam, lParam);
}

void MainWindow::onClose(HWND hWnd) {
	PostQuitMessage(0);
}

void MainWindow::onSize(HWND hWnd, WPARAM nType, int width, int height) {
}

void MainWindow::onPaint(HWND hWnd) {
	WMPainterRAII wmpraii(hWnd);
#ifdef DOUBLEBUFFERING
	Painter p(CreateCompatibleDC(wmpraii.painter().hdc()));
#else
	Painter &p = wmpraii.painter();
#endif

	Rect rc;
	GetClientRect(hWnd, rc.prect());
#ifdef DOUBLEBUFFERING
	// note: do not DeleteObject(SelectObject(..., CreateCompatibleBitmap(...)))
	//       even if you don't need old bitmap, it cause some GDI objects leak...
	HGDIOBJ hbmold = SelectObject(p.hdc(), CreateCompatibleBitmap(wmpraii.painter().hdc(), rc.width(), rc.height()));
#endif
	p.setSize(rc.size());
#ifdef DOUBLEBUFFERING
	{
		// erase bg by black.
		PenBrushRAII pbraii(p, 3, RGB(0, 0, 0), PS_SOLID, false, 0);
		p.fillRect(rc);
	}
#endif

	if(m_scene) {
		m_scene->onDraw(p);

		// debug
		switch(m_scene->result()) {
		case SR_SUCCESS:
			g_font.draw(p, Point(10, 10), 5.0f, 0, L"○");
			break;
		case SR_FAILED:
			g_font.draw(p, Point(10, 10), 5.0f, 0, L"×");
			break;
		default:
			// nothing
			break;
		}
	}

#ifdef DOUBLEBUFFERING
	BitBlt(wmpraii.painter().hdc(), rc.left(), rc.top(), rc.right(), rc.bottom(), p.hdc(), 0, 0, SRCCOPY);
	DeleteObject(SelectObject(p.hdc(), hbmold));
#endif
}

void MainWindow::onMousemove(HWND hWnd, int x, int y, UINT keys) {
	m_curpos = xy2rip(x, y);

	if(m_scene) {
		m_scene->onMousemove(m_curpos);
	}
}

void MainWindow::onLbuttondown(HWND hWnd, BOOL isdouble, int x, int y, UINT keys) {
	SetCapture(hWnd);

	m_curpos = xy2rip(x, y);

	if(m_scene) {
		m_scene->onButtonDown(LBUTTON, m_curpos);
	}
}

void MainWindow::onLbuttonup(HWND hWnd, int x, int y, UINT keys) {
	ReleaseCapture();

	if(m_scene) {
		m_scene->onButtonUp(LBUTTON, m_curpos);
	}
}

void MainWindow::onTimer(HWND hWnd, int nIDEvent) {
	// called about 1/60 sec.
#ifdef DOUBLEBUFFERING
	InvalidateRect(hWnd, NULL, FALSE);
#else
	InvalidateRect(hWnd, NULL, TRUE);
#endif

	if(m_scene) {
		m_scene->onTimer();

		Scene *next = m_scene->nextScene();
		if(next) {
			delete m_scene;
			m_scene = next;
		}
	}
}

void MainWindow::onRbuttondown(HWND hWnd, BOOL isdouble, int x, int y, UINT keys) {
	SetCapture(hWnd);

	m_curpos = xy2rip(x, y);

	if(m_scene) {
		m_scene->onButtonDown(RBUTTON, m_curpos);
	}
}

void MainWindow::onRbuttonup(HWND hWnd, int x, int y, UINT keys) {
	ReleaseCapture();

	if(m_scene) {
		m_scene->onButtonUp(RBUTTON, m_curpos);
	}
}

void MainWindow::onKey(HWND hWnd, UINT vk, BOOL isdown, int rptcnt, UINT states) {
	if(m_scene) {
		m_scene->onKey(vk, isdown);
	}

#ifdef _DEBUG
	if(vk == VK_PAUSE) {
		wmKillAllWave();
		delete m_scene;
		m_scene = new SelectorScene(1);
	}
#endif
}

void MainWindow::onChar(HWND hWnd, TCHAR ch, int rptcnt) {
	if(m_scene) {
		m_scene->onChar(ch);
	}
}

void MainWindow::onMbuttondown(HWND hWnd, BOOL isdouble, int x, int y, UINT keys) {
	SetCapture(hWnd);

	m_curpos = xy2rip(x, y);

	if(m_scene) {
		m_scene->onButtonDown(MBUTTON, m_curpos);
	}
}

void MainWindow::onMbuttonup(HWND hWnd, int x, int y, UINT keys) {
	ReleaseCapture();

	if(m_scene) {
		m_scene->onButtonUp(MBUTTON, m_curpos);
	}
}

void MainWindow::onMousewheel(HWND hWnd, USHORT vk, int delta, int x, int y) {
	if(m_scene) {
		m_scene->onMousewheel(m_curpos, static_cast<float>(delta) / WHEEL_DELTA);
	}
}

Point MainWindow::xy2rip(int x, int y) {
	Rect rc;
	GetClientRect(m_hWnd, rc.prect());
	return Point(x * RIP_W / rc.width(), y * RIP_H / rc.height());
}
