class Painter {
	friend class PenBrushRAII; // accessing m_hDC, m_hBrush.
	friend class PreserveViewrgnRAII; // accessing m_hDC.
public:
// ctors
	Painter(HDC hDC, bool deleteOnDtor=true);
// dtors
	~Painter();
// methods
	void setSize(Size sz);
	void setViewport(Rect rc);
	void resetViewport();
	void appendViewmask(Rect rc);
	void line(Point p1, Point p2);
	void text(Point p, LPCTSTR text);
	void frameRect(Rect rc);
	void fillRect(Rect rc);

	// low-level function; do not use. ... or use with care when that is only way.
	HDC hdc() { return m_hDC; }
private:
// variables
	HDC m_hDC;
	Size m_sz;
	HGDIOBJ m_hOrgPen;
	HBRUSH m_hBrush;
	HGDIOBJ m_hOrgBrush;
	bool m_deleteOnDtor;
// internal functions
	Point rip2pt(Point p);
};

class PenBrushRAII {
public:
// ctors
	explicit PenBrushRAII(Painter &p);
	PenBrushRAII(Painter &p, int width, COLORREF color, int penstyle, bool isHatchBrush, int brushstyle);
// dtors
	~PenBrushRAII();
// methods
	void on(int width, COLORREF color, int penstyle, bool isHatchBrush, int brushstyle);
	void off();
private:
// variables
	Painter &m_painter;
	HGDIOBJ m_hOrgSpPen;
	HGDIOBJ m_hOrgSpBrush;
	bool m_on;
};

class PreserveViewrgnRAII {
public:
// ctors
	explicit PreserveViewrgnRAII(Painter &p);
// dtors
	~PreserveViewrgnRAII();
private:
	Painter &m_painter;
	HRGN m_orgrgn;
};

class WMPainterRAII {
public:
// ctors
	explicit WMPainterRAII(HWND hWnd);
// dtors
	~WMPainterRAII();
// methods
	Painter &painter() { return *m_painter; }
private:
	HWND m_hWnd;
	PAINTSTRUCT m_ps;

	// XXX: cannot use "Painter" member instead of "Painter*"?
	//      some GDI objects leaked when I used it.
	Painter *m_painter; // use pointer and new/delete to control Painter's lifetime strictly.
};

extern const int RIP_W;
extern const int RIP_H;
