class Painter;

enum MouseButton {
	NONE,
	LBUTTON,
	MBUTTON,
	RBUTTON,
};

enum SceneResult {
	SR_PLAYING,
	SR_SUCCESS,
	SR_FAILED,
};

// note: L/M/R buttons multi-presssing: only first button is valid at one moment.
//       other buttons are ignored.
//       e.g. Ldown->Mdown->drag->Lup->Mup
//          =>Ldown->Ldrag->Lup (M is ignored)
// note: assuming vk=0 is not used.
class Scene { // abstract class
protected:
	explicit Scene(int level) {
		m_button = NONE;
		m_vk = 0;

		m_nextScene = NULL;
		m_result = SR_PLAYING;

		m_level = level;

		m_beginTime = systime();
	}
public:
	virtual ~Scene() {}
public:
// methods
	void onButtonDown(enum MouseButton btn, Point curpos);
	void onButtonUp(enum MouseButton btn, Point curpos);
	void onMousemove(Point curpos);
	void onDraw(Painter &p) { draw(p); }
	// mainly for FontEditorScene...
	void onMousewheel(Point curpos, float delta);
	void onKey(UINT vk, BOOL isDown);
	void onChar(TCHAR ch) { moji(ch); }
	void onTimer() { tick(); }

	Scene *nextScene() { return m_nextScene; }
	enum SceneResult result() { return m_result; }

	virtual float timelimit() { return 0.0f; /* 0.0f = None */ }
// callbacks
	virtual void tick() {}
	virtual void draw(Painter &p) {}
	virtual void button(enum MouseButton btn, Point curpos, bool isdown) {}
	virtual void wheel(Point curpos, float delta) {}
	virtual void drag(enum MouseButton btn, Point curpos) {}
	virtual void hover(Point curpos) {}
	virtual void key(UINT vk, bool isdown) {}
	virtual void moji(TCHAR ch) {}
// common-variables
	UINT m_vk;
	enum MouseButton m_button;

	int m_level;
// common-internal-functions
	bool isKey(UINT vk) { return (GetKeyState(vk) & 0x8000) != 0; }
	ULARGE_INTEGER ft2uli(FILETIME &ft) {
		ULARGE_INTEGER uli;
		uli.LowPart = ft.dwLowDateTime;
		uli.HighPart = ft.dwHighDateTime;
		return uli;
	}
	ULARGE_INTEGER systime() {
		FILETIME now;
		GetSystemTimeAsFileTime(&now);
		return ft2uli(now);
	}
	float getsec() {
		ULARGE_INTEGER now = systime();
		// systime() = 100 nano second precision.
		return (now.QuadPart - m_beginTime.QuadPart) / 10000000.0f;
	}
	void setNextScene(Scene* scene) {
		m_nextScene = scene;
	}
	void setResult(enum SceneResult result) {
		m_result = result;
	}
private:
// variables
	ULARGE_INTEGER m_beginTime;
	Scene *m_nextScene;
	enum SceneResult m_result;
};

class SceneBuilder;

#include <iterator> // debug

class SceneManager {
public:
//methods
	std::vector<SceneBuilder*> &allbuilders() { return m_builders; }
	std::vector<SceneBuilder*> &builders() { return m_nondevbuilders; }
	void addBuilder(SceneBuilder *builder) {
		//debug(_T("addBuilder with %p, m_builders.size=%d"), builder, m_builders.size());
		//m_builders.push_back(builder);
		m_nondevbuilders.push_back(builder);
		/*
		if(!builder->isDevScene()) {
			m_nondevbuilders.push_back(builder);
		}
		//*/
		//debug(_T("->m_builders.size=%d"), m_builders.size());
	}
	// XXX: I am loser. should handled in addBuilder(). currently MainWindow::create() handles.
	void addDevBuilder(SceneBuilder *builder) {
		m_builders.push_back(builder);
	}

/*
	unsigned int builderSize() { return m_builders.size(); }
	SceneBuilder *builderAt(unsigned int idx) {
		if(m_builders.size() <= idx) {
			return NULL;
		}

		std::vector<SceneBuilder*>::iterator it = m_builders.begin();
		std::advance(it, idx);

		return *it;
	}
//*/
private:
// variables
	std::vector<SceneBuilder*> m_builders;
	std::vector<SceneBuilder*> m_nondevbuilders;
};

extern SceneManager g_sceneManager;

class SceneBuilder {
protected:
	SceneBuilder() {
		// XXX: depends link order. is this bad idea??
		//g_sceneManager.addBuilder(this);
		g_sceneManager.addDevBuilder(this);
	}
public:
// abstract methods
	virtual LPCWSTR sceneName() = 0;
	virtual Scene *newScene(int level) = 0;
	virtual bool isDevScene() = 0;
};
