#include <tchar.h>
#include <windows.h>

#include <vector>
#include <map>

#include "support.h"
#include "zipfs/zipfs.h"

#include "Scene.h"
#include "Painter.h"
#include "FontGlyph.h"

#include "Globals.h"

#include "SelectorScene.h"


static SelectorSceneBuilder g_selectorSceneBuilder;

SelectorScene::SelectorScene(int level) : Scene(level) {
	std::vector<SceneBuilder*> &builders = g_sceneManager.allbuilders();

	for(std::vector<SceneBuilder*>::iterator it = builders.begin(); it != builders.end(); it++) {
		if(*it == &g_selectorSceneBuilder) {
			continue;
		}
		m_selectableBuilders.push_back(*it);
	}

	m_level = 1;
}

void SelectorScene::draw(Painter &p) {
	wchar_t buf[256];

	g_font.draw(p, Point(10, 10), 0.3f, 0, L"Scene Selector");

	swprintf(buf, L"%d scenes.", m_selectableBuilders.size());
	g_font.draw(p, Point(10, 30), 0.3f, 0, buf);

	swprintf(buf, L"Level=%d", m_level);
	g_font.draw(p, Point(500, 10), 0.3f, 0, buf);

	Point pt(30, 70);
	for(std::vector<SceneBuilder*>::iterator it = m_selectableBuilders.begin(); it != m_selectableBuilders.end(); it++) {
		swprintf(buf, L"%s%s", (*it)->sceneName(), (*it)->isDevScene() ? L" (dev)" : L"");
		g_font.draw(p, pt, 0.3f, 0, buf); // XXX: only unicode build

		Rect rc = pt.rect(Size(600, 20));
		if(rc.has(m_curpos)) {
			p.frameRect(rc);
		}

		pt.sety(pt.y() + 20);
	}
}

void SelectorScene::button(enum MouseButton btn, Point curpos, bool isdown) {
	m_curpos = curpos;

	if(!isdown) {
		return;
	}

	if(m_button == LBUTTON) {
		Point pt(30, 70);
		for(std::vector<SceneBuilder*>::iterator it = m_selectableBuilders.begin(); it != m_selectableBuilders.end(); it++) {
			Rect rc = pt.rect(Size(600, 20));
			if(rc.has(m_curpos)) {
				setNextScene((*it)->newScene(m_level));
				break;
			}
			pt.sety(pt.y() + 20);
		}
	}
}

void SelectorScene::moji(TCHAR ch) {
	switch(ch) {
	case '1':
	case '2':
	case '3':
		m_level = ch - '1' + 1;
		break;
	default:
		// nothing
		break;
	}
}
