#include <tchar.h>
#include <windows.h>

#include <vector>
#include <map>

#include "support.h"
#include "zipfs/zipfs.h"
#include "wavemix/wavemix.h"
#include "zipwave.h"

#include "Scene.h"
#include "Painter.h"
#include "FontGlyph.h"

#include "Globals.h"

#include "SheepCountGameScene.h"

enum SheepGlyphs {
	SG_SHEEP1   = 0xE000,
	SG_SHEEP2   = 0xE001,
	SG_SAKU     = 0xE002,
	SG_CLOUD    = 0xE003,
	SG_LAND     = 0xE004,
	SG_SHEEPBAD = 0xE005,
};

const int sheepbasey = 225;

static SheepCountGameSceneBuilder g_sheepCountGameSceneBuilder;

SheepCountGameScene::SheepCountGameScene(int level) : Scene(level) {
	m_sheepx = 640;
	m_sheepy = 225;
	m_sheepvy = 0.0f;
	m_sheepbad = false;
	m_sakux = 200;
	m_baatime = 0.0f;

	switch(level) {
	case 1:
		m_sheepvx = -5;
		m_sheepjumpvy = -12.0f;
		m_sheepfallay = 0.6f;
		break;
	case 2:
		m_sheepvx = -7;
		m_sheepjumpvy = -15.0f;
		m_sheepfallay = 1.2f;
		break;
	default: // ?
		// FALLTHROUGH
	case 3:
		m_sheepvx = -9;
		m_sheepjumpvy = -18.0f;
		m_sheepfallay = 1.8f;
		break;
	}

	Glyph *g = g_font.glyph(SG_CLOUD);
	for(unsigned int i = 0; i < _countof(m_cloud); i++) {
		m_cloud[i] = Point(randr(-g->bounding().cx(), 640 + 100), randr(0, 85));
		m_cloudspeed[i] = randr(1, 3);
	}

	m_boing = unzLoadWave("pluck3.wav");
	m_crash = unzLoadWave("warpy-boing.wav");
	m_baa = unzLoadWave("sheep-baa.wav");
}

SheepCountGameScene::~SheepCountGameScene() {
	wmUnloadWave(m_boing);
	wmUnloadWave(m_crash);
	wmUnloadWave(m_baa);
}

void SheepCountGameScene::tick() {
	Glyph *g;

	if(!m_sheepbad) {
		m_sheepx += m_sheepvx;

		if(m_sakux - 100 <= m_sheepx && m_sheepx <= m_sakux + 50 && 200 < m_sheepy) {
			m_sheepbad = true;
			m_baatime = getsec() + 0.8f;
			wmPlayWave(m_crash);
			setResult(SR_FAILED);
		}
		g = g_font.glyph(SG_SHEEP1);
		if(result() == SR_PLAYING &&  m_sheepx < m_sakux + 0/*TODO*/ - g->bounding().cx()) {
			wmPlayWave(m_baa);
			setResult(SR_SUCCESS);
		}
	}

	if(m_sheepbad && m_baatime != 0.0f && m_baatime < getsec()) {
		wmPlayWave(m_baa);
		m_baatime = 0.0f;
	}

	if(m_sheepy < sheepbasey || m_sheepvy != 0.0f) {
		m_sheepy += m_sheepvy;
		m_sheepvy += m_sheepfallay;

		if(sheepbasey <= m_sheepy) {
			m_sheepy = sheepbasey;
			m_sheepvy = 0;
		}
	}

	g = g_font.glyph(SG_CLOUD);
	for(unsigned int i = 0; i < _countof(m_cloud); i++) {
		m_cloud[i].setx(m_cloud[i].x() - m_cloudspeed[i]);
		if(m_cloud[i].x() < -g->bounding().cx()) {
			m_cloud[i] = Point(randr(640, 640 + 100), randr(0, 85));
			m_cloudspeed[i] = randr(1, 3);
		}
	}
}

void SheepCountGameScene::draw(Painter &p) {
	Glyph *g;

	float sec = getsec();
	float rem = sec - (int)sec;
	g = g_font.glyph(m_sheepbad ? SG_SHEEPBAD : (rem < 0.5f ? SG_SHEEP1 : SG_SHEEP2));
	g->draw(p, Point(m_sheepx, m_sheepy), 1.0f, GLYPHRAND);

	g = g_font.glyph(SG_SAKU);
	g->draw(p, Point(m_sakux, 0), 1.0f, GLYPHRAND);

	g = g_font.glyph(SG_CLOUD);
	for(unsigned int i = 0; i < _countof(m_cloud); i++) {
		g->draw(p, m_cloud[i], 1.0f, GLYPHRAND);
	}

	g = g_font.glyph(SG_LAND);
	g->draw(p, Point(0, 0), 1.0f, GLYPHRAND);
}

void SheepCountGameScene::button(enum MouseButton btn, Point curpos, bool isdown) {
	if(!isdown) {
		return;
	}

	if(!m_sheepbad && m_sheepy == sheepbasey) {
		m_sheepvy = m_sheepjumpvy;
		wmPlayWave(m_boing);
	}
}
