#include <tchar.h>
#include <windows.h>

#include <math.h>

#include <vector>
#include <map>

#include "support.h"
#include "zipfs/zipfs.h"
#include "wavemix/wavemix.h"
#include "zipwave.h"

#include "Scene.h"
#include "Painter.h"
#include "FontGlyph.h"

#include "Globals.h"

#include "TitleScene.h"
#include "MainGameScene.h"


enum TitleGlyphs {
	TG_TITLE = 0xE062,
};

static const float CLICKSTARTDUR = 2.0f;
static const float CLICKSTARTFADE = 0.5f; // "% 2"'ed
static const float STARTWAIT = 0.3f;
static const float STARTFADE = 0.5f;
static const float STARTFADEWAIT = 0.3f;

static TitleSceneBuilder g_titleSceneBuilder;

TitleScene::TitleScene(int level) : Scene(level) {
	m_acceptedsec = 0.0f;
	m_ticked = false;

	m_wbgm = unzLoadWave("st017.wav");
	m_wclick = unzLoadWave("meka_ta_sui04.wav");
}

TitleScene::~TitleScene() {
	wmKillWave(m_wbgm);

	wmUnloadWave(m_wbgm);
	wmUnloadWave(m_wclick);
}

void TitleScene::tick() {
	if(!m_ticked) {
		m_ticked = true;

		wmPlayWave(m_wbgm);
	}

	// TODO: fadeout bgm
	if(m_acceptedsec != 0.0f && m_acceptedsec + STARTWAIT + STARTFADE + STARTFADEWAIT < getsec()) {
		setNextScene(new MainGameScene(0));
	}
}

void TitleScene::draw(Painter &p) {
	float sec = getsec();
	PenBrushRAII pbraii(p);
	float f1 = 1.0f;

	if(m_acceptedsec != 0.0f && m_acceptedsec + STARTWAIT < getsec()) {
		f1 = std::max(std::min(1.0f - (sec - (m_acceptedsec + STARTWAIT)) / STARTFADE, 1.0f), 0.0f);
		int fade = static_cast<int>(f1 * 255);
		pbraii.on(3, RGB(fade, fade, fade), PS_SOLID, false, 0);
	}

	Glyph *g = g_font.glyph(TG_TITLE);
	g->draw(p, Point(0, 0), 1.0f, GLYPHRAND);

	if(CLICKSTARTDUR < sec) {
		float remsec = sec - CLICKSTARTDUR;
		float f2 = remsec / CLICKSTARTFADE;
		if(static_cast<int>(f2) % 2 == 0) {
			// fade-in
			f2 = std::max(std::min(f2 - floorf(f2), 1.0f), 0.0f);
		} else {
			// fade-out
			f2 = std::max(std::min(1.0f - (f2 - floorf(f2)), 1.0f), 0.0f);
		}
		int fade = static_cast<int>(f2 * f1/*total fade*/ * 255);
		PenBrushRAII pbraii2(p, 3, RGB(fade, fade, fade), PS_SOLID, false, 0);
		Point off(320, 430);
		float mag = 0.7f;
		LPCWSTR str = L"クリックですた～と";
		Size sz = g_font.bounding(mag, str);
		off -= sz.asPoint() / 2;
		g_font.draw(p, off, mag, GLYPHRAND, str);
	}
}

void TitleScene::button(enum MouseButton btn, Point curpos, bool isdown) {
	if(!isdown) {
		return;
	}

	if(m_acceptedsec == 0.0f) {
		m_acceptedsec = getsec();
		wmPlayWave(m_wclick);
	}
}

//void TitleScene::drag(enum MouseButton btn, Point curpos) {}
//void TitleScene::hover(Point curpos) {}
