#include <tchar.h>
#include <windows.h>

#include <vector>
#include <map>

#include "support.h"
#include "zipfs/zipfs.h"
#include "wavemix/wavemix.h"
#include "zipwave.h"

#include "Scene.h"
#include "Painter.h"
#include "FontGlyph.h"

#include "Globals.h"

#include "ToothbrushGameScene.h"


enum TemplateGlyphs {
	TG_FACE = 0xE050,
	TG_BRUSH = 0xE051,
	TG_BRUSHTEXT = 0xE052,
	TG_CLEANANIM1 = 0xE053,
	TG_CLEANANIM2,
	TG_CLEANANIM3,
	TG_CLEANANIM4,
	TG_FACEFIX = 0xE057,
};

static const int BRUSHY = 283;
static const int BRUSHMINX = -140;
static const int BRUSHMAXX = 0;

static ToothbrushGameSceneBuilder g_toothbrushGameSceneBuilder;

ToothbrushGameScene::ToothbrushGameScene(int level) : Scene(level) {
	m_plaque = (BRUSHMAXX - BRUSHMINX) * 2 * level;
	m_brushx = BRUSHMINX;
	m_lastmove = -5.0f; // less than 0.0f enough to first sound.
	m_completesec = 0.0f;

	m_wbrush1 = unzLoadWave("se_maoudamashii_se_ignition02.wav");
	m_wbrush2 = unzLoadWave("se_maoudamashii_se_ignition03.wav");
	m_wclean = unzLoadWave("se_maoudamashii_effect11.wav");
}

ToothbrushGameScene::~ToothbrushGameScene() {
	wmUnloadWave(m_wbrush1);
	wmUnloadWave(m_wbrush2);
	wmUnloadWave(m_wclean);
}

//void ToothbrushGameScene::tick() {}

void ToothbrushGameScene::draw(Painter &p) {
	Glyph *g;

	{
		PreserveViewrgnRAII pvraii(p);

		Size sz = g_font.glyph(TG_BRUSH)->bounding();
		Rect rc = Point(m_brushx, BRUSHY).rect(sz);
		rc.setLeft(std::min(0L, rc.left()));
		p.appendViewmask(rc);

		g = g_font.glyph(TG_FACE);
		g->draw(p, Point(0, 0), 1.0f, GLYPHRAND);
	}

	// XXX: dirty hack!
	g = g_font.glyph(TG_FACEFIX);
	g->draw(p, Point(0, 0), 1.0f, GLYPHRAND);

	g = g_font.glyph(TG_BRUSH);
	g->draw(p, Point(m_brushx, BRUSHY), 1.0f, GLYPHRAND);
	g = g_font.glyph(TG_BRUSHTEXT);
	g->draw(p, Point(m_brushx, BRUSHY), 1.0f, GLYPHRAND);

	if(m_completesec != 0.0f) {
		float sec = getsec() - m_completesec;
		if(sec < 0.2f * 4) {
			int frame = static_cast<int>(sec / 0.2f);
			g = g_font.glyph(TG_CLEANANIM1 + frame);
			g->draw(p, Point(0, 0), 1.0f, GLYPHRAND);
		}
	}
}

void ToothbrushGameScene::button(enum MouseButton btn, Point curpos, bool isdown) {
	if(!isdown) {
		return;
	}

	m_precurpos = curpos;
}

void ToothbrushGameScene::drag(enum MouseButton btn, Point curpos) {
	int diffx = curpos.x() - m_precurpos.x();
	if(diffx == 0) { // only y-axis moving
		return;
	}

	int movesgn = sgn(diffx);
	if(0 < movesgn) {
		diffx = std::min(diffx, BRUSHMAXX - m_brushx);
	} else { // movesgn < 0
		diffx = -std::min(-diffx, m_brushx - BRUSHMINX);
	}
	if(diffx == 0) { // could not move brush more
		return;
	}
	m_brushx += diffx;

	if(movesgn != m_lastmovesgn) {
		m_lastmove = -5.0f; // less than 0.0f enough to first sound.
	}
	if(m_lastmove + 0.5f < getsec()) {
		if(0 < diffx) {
			wmPlayWave(m_wbrush1);
		} else {
			wmPlayWave(m_wbrush2);
		}
	}

	m_plaque = std::max(0, m_plaque - abs(diffx));
	if(result() == SR_PLAYING && m_plaque <= 0) {
		setResult(SR_SUCCESS);
		wmPlayWave(m_wclean);
		m_completesec = getsec();
	}

	m_lastmovesgn = movesgn;
	m_lastmove = getsec();
	m_precurpos = curpos;
}

//void ToothbrushGameScene::hover(Point curpos) {}
