// from ake2009...
//#define ZIPFS // using zip file
//#define ZIPRESOURCE // using resource embedded zip, ZIPFS must be defined

#include <tchar.h>
#include "zipfs.h"
#ifndef _countof
#define _countof(a) (sizeof(a)/sizeof((a)[0]))
#endif
#include "ffres.h"
#ifndef ZIPFS
#include <stdio.h>
#endif

BYTE *unzReadWhole(unzFile *zip, const char *fname, long *size)
{
	BYTE *buf;
	long s;
#ifdef ZIPFS
	unz_file_info info;

	if(unzLocateFile(zip, fname, 0) != UNZ_OK) {
		TCHAR msgbuf[256];
#ifdef _UNICODE
		TCHAR Fname[256];
		MultiByteToWideChar(CP_UTF8, 0, fname, -1, Fname, _countof(Fname) - 1);
#define fname Fname
#endif
		// note: using _stprintf is dangerous, both buffer-overflow and sprintf/swprintf type signature difference.
		//       _stprintf_s is defined in MinGW-w64 but undefined in MinGW (mingw32-w32api-dev 4.0.3-1)
		//       StringCchPrintf too.
		//       picked up _sntprintf to usable in both MinGW(32bit) and MinGW-w64.
		_sntprintf(msgbuf, _countof(msgbuf), _T("unzReadWhole: File [%s] not found in zipfile"), fname);
#ifdef _UNICODE
#undef fname
#endif
		MessageBox(NULL, msgbuf, _T("Fatal"), MB_ICONERROR | MB_OK);
		return NULL;
	}

	unzGetCurrentFileInfo(zip, &info, NULL, 0, NULL, 0, NULL, 0);
	s = info.uncompressed_size;
	buf = malloc(s + 1);
	if(size)
		*size = s;
	unzOpenCurrentFile(zip);
	unzReadCurrentFile(zip, buf, s);
	buf[s] = '\0';
	unzCloseCurrentFile(zip);
#else
	char fnbuf[256];
	FILE *fp;
	sprintf(fnbuf, "data\\%s", fname);
	if(!(fp = fopen(fnbuf, "rb"))) {
		TCHAR msgbuf[256];
#ifdef _UNICODE
		TCHAR Fnbuf[256];
		MultiByteToWideChar(CP_UTF8, 0, fnbuf, -1, Fnbuf, _countof(Fnbuf) - 1);
#define fnbuf Fnbuf
#endif
		// note: this too. (see above _sntprintf comment)
		_sntprintf(msgbuf, _countof(msgbuf), _T("unzReadWhole: File [%s] not found in filesystem"), fnbuf);
#ifdef _UNICODE
#undef fnbuf
#endif
		MessageBox(NULL, msgbuf, _T("Fatal"), MB_ICONERROR | MB_OK);
		return NULL;
	}
	fseek(fp, 0, SEEK_END);
	s = ftell(fp);
	if(size)
		*size = s;
	buf = malloc(s + 1);
	fseek(fp, 0, SEEK_SET);
	fread(buf, s, 1, fp);
	buf[s] = '\0';
	fclose(fp);
#endif

	return buf;
}

#ifdef ZIPFS_SDLMIXER
Mix_Chunk *Mix_LoadWAVfromzip(unzFile *zip, const char *fname)
{
	Mix_Chunk *wav;
	BYTE *buf;
	long size;

	if(!(buf = unzReadWhole(zip, fname, &size)))
		return NULL;

	wav = Mix_LoadWAV_RW(SDL_RWFromMem(buf, size), 1);

	free(buf);

	return wav;
}
#endif

#ifdef ZIPFS_GL
unsigned int pngBindfromzip(unzFile *zip, const char *fname, int mipmap, int trans, pngInfo *info, int wrapst, int minfilter, int magfilter)
{
	unsigned int ret;
	BYTE *buf;

	if(!(buf = unzReadWhole(zip, fname, NULL)))
	{
		MessageBox(NULL, _T("Corrupted EXE file, re-download please!\n\nInternal error: file not found."), _T("Fatal"), MB_ICONERROR | MB_OK);
		exit(0);
	}

	ret = pngBindX(buf, mipmap, trans, info, wrapst, minfilter, magfilter);

	free(buf);

	return ret;
}
#endif

unzFile *openres(void)
{
	unzFile *zip = NULL;
#ifdef ZIPFS
#ifdef ZIPRESOURCE
	if(!(zip = unzOpen2("DATA", getResFileFuncDef())))
	{
		MessageBox(NULL, _T("Corrupted EXE file, re-download please!\n\nInternal error: resource not found."), _T("Fatal"), MB_ICONERROR | MB_OK);
		exit(0);
	}
#else
	if(!(zip = unzOpen("data.zip")))
	{
		MessageBox(NULL, _T("Corrupted EXE file, re-download please!\n\nInternal error: zip not found."), _T("Fatal"), MB_ICONERROR | MB_OK);
		exit(0);
	}
#endif
#else
	//SetCurrentDirectory("data");
#endif

	return zip;
}

void closeres(unzFile *zip)
{
#ifdef ZIPFS
	unzClose(zip);
#endif
}

/*
void loadres()
{
	unzFile *zip = openres();

	sndshot = Mix_LoadWAVfromzip(zip, "shot.wav");
	texjet = pngBindfromzip(zip, "jet.png", PNG_NOMIPMAP, PNG_ALPHA, NULL, GL_CLAMP, GL_NEAREST, GL_NEAREST);

	closeres(zip);
}
*/
